@@ -1,3 +1,4 @@
+# -*- coding: us-ascii -*-
 =begin
 = $RCSfile$ -- Generator for Makefile
 
@@ -11,11 +12,11 @@
   (See the file 'LICENCE'.)
 
 = Version
-  $Id: extconf.rb 36884 2012-09-03 01:14:26Z emboss $
+  $Id: extconf.rb 42141 2013-07-23 14:19:51Z naruse $
 =end
 
 require "mkmf"
-require_relative 'deprecation'
+require File.expand_path('../deprecation', __FILE__)
 
 dir_config("openssl")
 dir_config("kerberos")
@@ -143,10 +144,12 @@ if checking_for('OpenSSL version is 0.9.7 or later') {
   }
   have_header("openssl/ocsp.h")
 end
+have_struct_member("CRYPTO_THREADID", "ptr", "openssl/crypto.h")
 have_struct_member("EVP_CIPHER_CTX", "flags", "openssl/evp.h")
 have_struct_member("EVP_CIPHER_CTX", "engine", "openssl/evp.h")
 have_struct_member("X509_ATTRIBUTE", "single", "openssl/x509.h")
 have_macro("OPENSSL_FIPS", ['openssl/opensslconf.h']) && $defs.push("-DHAVE_OPENSSL_FIPS")
+have_macro("EVP_CTRL_GCM_GET_TAG", ['openssl/evp.h']) && $defs.push("-DHAVE_AUTHENTICATED_ENCRYPTION")
 
 Logging::message "=== Checking done. ===\n"
 
