@@ -2,7 +2,7 @@
 
   range.c -
 
-  $Author: akr $
+  $Author: nobu $
   created at: Thu Aug 19 17:46:47 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -11,18 +11,28 @@
 
 #include "ruby/ruby.h"
 #include "ruby/encoding.h"
+#include "internal.h"
+#include "id.h"
+
+#ifdef HAVE_FLOAT_H
+#include <float.h>
+#endif
+#include <math.h>
 
 VALUE rb_cRange;
-static ID id_cmp, id_succ, id_beg, id_end, id_excl;
-
-extern VALUE rb_struct_init_copy(VALUE copy, VALUE s);
+static ID id_cmp, id_succ, id_beg, id_end, id_excl, id_integer_p, id_div;
 
 #define RANGE_BEG(r) (RSTRUCT(r)->as.ary[0])
 #define RANGE_END(r) (RSTRUCT(r)->as.ary[1])
 #define RANGE_EXCL(r) (RSTRUCT(r)->as.ary[2])
 
 #define EXCL(r) RTEST(RANGE_EXCL(r))
-#define SET_EXCL(r,v) (RSTRUCT(r)->as.ary[2] = (v) ? Qtrue : Qfalse)
+static inline VALUE
+SET_EXCL(VALUE r, VALUE v)
+{
+    RSTRUCT_SET(r, 2, v);
+    return v ? Qtrue : Qfalse;
+}
 
 static VALUE
 range_failed(void)
@@ -54,8 +64,8 @@ range_init(VALUE range, VALUE beg, VALUE end, int exclude_end)
     }
 
     SET_EXCL(range, exclude_end);
-    RSTRUCT(range)->as.ary[0] = beg;
-    RSTRUCT(range)->as.ary[1] = end;
+    RSTRUCT_SET(range, 0, beg);
+    RSTRUCT_SET(range, 1, end);
 }
 
 VALUE
@@ -69,10 +79,10 @@ rb_range_new(VALUE beg, VALUE end, int exclude_end)
 
 /*
  *  call-seq:
- *     Range.new(start, end, exclusive=false)    => range
+ *     Range.new(begin, end, exclude_end=false)    -> rng
  *
- *  Constructs a range using the given <i>start</i> and <i>end</i>. If the third
- *  parameter is omitted or is <code>false</code>, the <i>range</i> will include
+ *  Constructs a range using the given +begin+ and +end+. If the +exclude_end+
+ *  parameter is omitted or is <code>false</code>, the +rng+ will include
  *  the end object; otherwise, it will be excluded.
  */
 
@@ -84,7 +94,7 @@ range_initialize(int argc, VALUE *argv, VALUE range)
     rb_scan_args(argc, argv, "21", &beg, &end, &flags);
     /* Ranges are immutable, so that they should be initialized only once. */
     if (RANGE_EXCL(range) != Qnil) {
-	rb_name_error(rb_intern("initialize"), "`initialize' called twice");
+	rb_name_error(idInitialize, "`initialize' called twice");
     }
     range_init(range, beg, end, RTEST(flags));
     return Qnil;
@@ -94,9 +104,12 @@ range_initialize(int argc, VALUE *argv, VALUE range)
 
 /*
  *  call-seq:
- *     rng.exclude_end?    => true or false
+ *     rng.exclude_end?    -> true or false
  *
- *  Returns <code>true</code> if <i>rng</i> excludes its end value.
+ *  Returns <code>true</code> if the range excludes its end value.
+ *
+ *     (1..5).exclude_end?     #=> false
+ *     (1...5).exclude_end?    #=> true
  */
 
 static VALUE
@@ -105,14 +118,28 @@ range_exclude_end_p(VALUE range)
     return EXCL(range) ? Qtrue : Qfalse;
 }
 
+static VALUE
+recursive_equal(VALUE range, VALUE obj, int recur)
+{
+    if (recur) return Qtrue; /* Subtle! */
+    if (!rb_equal(RANGE_BEG(range), RANGE_BEG(obj)))
+	return Qfalse;
+    if (!rb_equal(RANGE_END(range), RANGE_END(obj)))
+	return Qfalse;
+
+    if (EXCL(range) != EXCL(obj))
+	return Qfalse;
+    return Qtrue;
+}
+
 
 /*
  *  call-seq:
- *     rng == obj    => true or false
+ *     rng == obj    -> true or false
  *
- *  Returns <code>true</code> only if <i>obj</i> is a Range, has equivalent
- *  beginning and end items (by comparing them with <code>==</code>), and has
- *  the same <code>exclude_end?</code> setting as <i>rng</i>.
+ *  Returns <code>true</code> only if +obj+ is a Range, has equivalent
+ *  begin and end items (by comparing them with <code>==</code>), and has
+ *  the same #exclude_end? setting as the range.
  *
  *    (0..2) == (0..2)            #=> true
  *    (0..2) == Range.new(0,2)    #=> true
@@ -125,18 +152,10 @@ range_eq(VALUE range, VALUE obj)
 {
     if (range == obj)
 	return Qtrue;
-    if (!rb_obj_is_instance_of(obj, rb_obj_class(range)))
+    if (!rb_obj_is_kind_of(obj, rb_cRange))
 	return Qfalse;
 
-    if (!rb_equal(RANGE_BEG(range), RANGE_BEG(obj)))
-	return Qfalse;
-    if (!rb_equal(RANGE_END(range), RANGE_END(obj)))
-	return Qfalse;
-
-    if (EXCL(range) != EXCL(obj))
-	return Qfalse;
-
-    return Qtrue;
+    return rb_exec_recursive_paired(recursive_equal, range, obj, obj);
 }
 
 static int
@@ -145,10 +164,10 @@ r_lt(VALUE a, VALUE b)
     VALUE r = rb_funcall(a, id_cmp, 1, b);
 
     if (NIL_P(r))
-	return Qfalse;
+	return (int)Qfalse;
     if (rb_cmpint(r, a, b) < 0)
-	return Qtrue;
-    return Qfalse;
+	return (int)Qtrue;
+    return (int)Qfalse;
 }
 
 static int
@@ -158,23 +177,37 @@ r_le(VALUE a, VALUE b)
     VALUE r = rb_funcall(a, id_cmp, 1, b);
 
     if (NIL_P(r))
-	return Qfalse;
+	return (int)Qfalse;
     c = rb_cmpint(r, a, b);
     if (c == 0)
 	return (int)INT2FIX(0);
     if (c < 0)
-	return Qtrue;
-    return Qfalse;
+	return (int)Qtrue;
+    return (int)Qfalse;
 }
 
 
+static VALUE
+recursive_eql(VALUE range, VALUE obj, int recur)
+{
+    if (recur) return Qtrue; /* Subtle! */
+    if (!rb_eql(RANGE_BEG(range), RANGE_BEG(obj)))
+	return Qfalse;
+    if (!rb_eql(RANGE_END(range), RANGE_END(obj)))
+	return Qfalse;
+
+    if (EXCL(range) != EXCL(obj))
+	return Qfalse;
+    return Qtrue;
+}
+
 /*
  *  call-seq:
- *     rng.eql?(obj)    => true or false
+ *     rng.eql?(obj)    -> true or false
  *
- *  Returns <code>true</code> only if <i>obj</i> is a Range, has equivalent
- *  beginning and end items (by comparing them with #eql?), and has the same
- *  #exclude_end? setting as <i>rng</i>.
+ *  Returns <code>true</code> only if +obj+ is a Range, has equivalent
+ *  begin and end items (by comparing them with <code>eql?</code>),
+ *  and has the same #exclude_end? setting as the range.
  *
  *    (0..2).eql?(0..2)            #=> true
  *    (0..2).eql?(Range.new(0,2))  #=> true
@@ -187,34 +220,24 @@ range_eql(VALUE range, VALUE obj)
 {
     if (range == obj)
 	return Qtrue;
-    if (!rb_obj_is_instance_of(obj, rb_obj_class(range)))
+    if (!rb_obj_is_kind_of(obj, rb_cRange))
 	return Qfalse;
-
-    if (!rb_eql(RANGE_BEG(range), RANGE_BEG(obj)))
-	return Qfalse;
-    if (!rb_eql(RANGE_END(range), RANGE_END(obj)))
-	return Qfalse;
-
-    if (EXCL(range) != EXCL(obj))
-	return Qfalse;
-
-    return Qtrue;
+    return rb_exec_recursive_paired(recursive_eql, range, obj, obj);
 }
 
 static VALUE
 recursive_hash(VALUE range, VALUE dummy, int recur)
 {
-    unsigned long hash = EXCL(range);
+    st_index_t hash = EXCL(range);
     VALUE v;
 
-    if (recur) {
-        rb_raise(rb_eArgError, "recursive key for hash");
-    }
     hash = rb_hash_start(hash);
-    v = rb_hash(RANGE_BEG(range));
-    hash = rb_hash_uint(hash, NUM2LONG(v));
-    v = rb_hash(RANGE_END(range));
-    hash = rb_hash_uint(hash, NUM2LONG(v));
+    if (!recur) {
+	v = rb_hash(RANGE_BEG(range));
+	hash = rb_hash_uint(hash, NUM2LONG(v));
+	v = rb_hash(RANGE_END(range));
+	hash = rb_hash_uint(hash, NUM2LONG(v));
+    }
     hash = rb_hash_uint(hash, EXCL(range) << 24);
     hash = rb_hash_end(hash);
 
@@ -223,17 +246,17 @@ recursive_hash(VALUE range, VALUE dummy, int recur)
 
 /*
  * call-seq:
- *   rng.hash    => fixnum
+ *   rng.hash    -> fixnum
  *
- * Generate a hash value such that two ranges with the same start and
- * end points, and the same value for the "exclude end" flag, generate
- * the same hash value.
+ * Compute a hash-code for this range. Two ranges with equal
+ * begin and end points (using <code>eql?</code>), and the same
+ * #exclude_end? value will generate the same hash-code.
  */
 
 static VALUE
 range_hash(VALUE range)
 {
-    return rb_exec_recursive(recursive_hash, range, 0);
+    return rb_exec_recursive_outer(recursive_hash, range, 0);
 }
 
 static void
@@ -260,6 +283,24 @@ range_each_func(VALUE range, VALUE (*func) (VALUE, void *), void *arg)
     }
 }
 
+static VALUE
+sym_step_i(VALUE i, void *arg)
+{
+    VALUE *iter = arg;
+
+    if (FIXNUM_P(iter[0])) {
+	iter[0] -= INT2FIX(1) & ~FIXNUM_FLAG;
+    }
+    else {
+	iter[0] = rb_funcall(iter[0], '-', 1, INT2FIX(1));
+    }
+    if (iter[0] == INT2FIX(0)) {
+	rb_yield(rb_str_intern(i));
+	iter[0] = iter[1];
+    }
+    return Qnil;
+}
+
 static VALUE
 step_i(VALUE i, void *arg)
 {
@@ -278,33 +319,68 @@ step_i(VALUE i, void *arg)
     return Qnil;
 }
 
-extern int ruby_float_step(VALUE from, VALUE to, VALUE step, int excl);
+static int
+discrete_object_p(VALUE obj)
+{
+    if (rb_obj_is_kind_of(obj, rb_cTime)) return FALSE; /* until Time#succ removed */
+    return rb_respond_to(obj, id_succ);
+}
+
+static VALUE
+range_step_size(VALUE range, VALUE args, VALUE eobj)
+{
+    VALUE b = RANGE_BEG(range), e = RANGE_END(range);
+    VALUE step = INT2FIX(1);
+    if (args) {
+	step = RARRAY_AREF(args, 0);
+	if (!rb_obj_is_kind_of(step, rb_cNumeric)) {
+	    step = rb_to_int(step);
+	}
+    }
+    if (rb_funcall(step, '<', 1, INT2FIX(0))) {
+	rb_raise(rb_eArgError, "step can't be negative");
+    }
+    else if (!rb_funcall(step, '>', 1, INT2FIX(0))) {
+	rb_raise(rb_eArgError, "step can't be 0");
+    }
+
+    if (rb_obj_is_kind_of(b, rb_cNumeric) && rb_obj_is_kind_of(e, rb_cNumeric)) {
+	return ruby_num_interval_step_size(b, e, step, EXCL(range));
+    }
+    return Qnil;
+}
 
 /*
  *  call-seq:
- *     rng.step(n=1) {| obj | block }    => rng
+ *     rng.step(n=1) {| obj | block }    -> rng
+ *     rng.step(n=1)                     -> an_enumerator
  *
- *  Iterates over <i>rng</i>, passing each <i>n</i>th element to the block. If
- *  the range contains numbers, <i>n</i> is added for each iteration.  Otherwise
- *  <code>step</code> invokes <code>succ</code> to iterate through range
- *  elements. The following code uses class <code>Xs</code>, which is defined
- *  in the class-level documentation.
+ *  Iterates over the range, passing each <code>n</code>th element to the block.
+ *  If begin and end are numeric, +n+ is added for each iteration.
+ *  Otherwise <code>step</code> invokes <code>succ</code> to iterate through
+ *  range elements.
  *
- *     range = Xs.new(1)..Xs.new(10)
- *     range.step(2) {|x| puts x}
- *     range.step(3) {|x| puts x}
+ *  If no block is given, an enumerator is returned instead.
+ *
+ *    range = Xs.new(1)..Xs.new(10)
+ *    range.step(2) {|x| puts x}
+ *    puts
+ *    range.step(3) {|x| puts x}
  *
  *  <em>produces:</em>
  *
- *      1 x
- *      3 xxx
- *      5 xxxxx
- *      7 xxxxxxx
- *      9 xxxxxxxxx
- *      1 x
- *      4 xxxx
- *      7 xxxxxxx
- *     10 xxxxxxxxxx
+ *     1 x
+ *     3 xxx
+ *     5 xxxxx
+ *     7 xxxxxxx
+ *     9 xxxxxxxxx
+ *
+ *     1 x
+ *     4 xxxx
+ *     7 xxxxxxx
+ *    10 xxxxxxxxxx
+ *
+ *  See Range for the definition of class Xs.
  */
 
 
@@ -313,7 +389,7 @@ range_step(int argc, VALUE *argv, VALUE range)
 {
     VALUE b, e, step, tmp;
 
-    RETURN_ENUMERATOR(range, argc, argv);
+    RETURN_SIZED_ENUMERATOR(range, argc, argv, range_step_size);
 
     b = RANGE_BEG(range);
     e = RANGE_END(range);
@@ -347,13 +423,22 @@ range_step(int argc, VALUE *argv, VALUE range)
 	}
 
     }
+    else if (SYMBOL_P(b) && SYMBOL_P(e)) { /* symbols are special */
+	VALUE args[2], iter[2];
+
+	args[0] = rb_sym_to_s(e);
+	args[1] = EXCL(range) ? Qtrue : Qfalse;
+	iter[0] = INT2FIX(1);
+	iter[1] = step;
+	rb_block_call(rb_sym_to_s(b), rb_intern("upto"), 2, args, sym_step_i, (VALUE)iter);
+    }
     else if (ruby_float_step(b, e, step, EXCL(range))) {
 	/* done */
     }
     else if (rb_obj_is_kind_of(b, rb_cNumeric) ||
 	     !NIL_P(rb_check_to_integer(b, "to_int")) ||
 	     !NIL_P(rb_check_to_integer(e, "to_int"))) {
-	ID op = EXCL(range) ? '<' : rb_intern("<=");
+	ID op = EXCL(range) ? '<' : idLE;
 	VALUE v = b;
 	int i = 0;
 
@@ -379,7 +464,7 @@ range_step(int argc, VALUE *argv, VALUE range)
 	else {
 	    VALUE args[2];
 
-	    if (!rb_respond_to(b, id_succ)) {
+	    if (!discrete_object_p(b)) {
 		rb_raise(rb_eTypeError, "can't iterate from %s",
 			 rb_obj_classname(b));
 	    }
@@ -391,6 +476,213 @@ range_step(int argc, VALUE *argv, VALUE range)
     return range;
 }
 
+#if SIZEOF_DOUBLE == 8 && defined(HAVE_INT64_T)
+union int64_double {
+    int64_t i;
+    double d;
+};
+
+static VALUE
+int64_as_double_to_num(int64_t i)
+{
+    union int64_double convert;
+    if (i < 0) {
+	convert.i = -i;
+	return DBL2NUM(-convert.d);
+    }
+    else {
+	convert.i = i;
+	return DBL2NUM(convert.d);
+    }
+}
+
+static int64_t
+double_as_int64(double d)
+{
+    union int64_double convert;
+    convert.d = fabs(d);
+    return d < 0 ? -convert.i : convert.i;
+}
+#endif
+
+static int
+is_integer_p(VALUE v)
+{
+    VALUE is_int = rb_check_funcall(v, id_integer_p, 0, 0);
+    return RTEST(is_int) && is_int != Qundef;
+}
+
+/*
+ *  call-seq:
+ *     rng.bsearch {|obj| block }  -> value
+ *
+ *  By using binary search, finds a value in range which meets the given
+ *  condition in O(log n) where n is the size of the range.
+ *
+ *  You can use this method in two use cases: a find-minimum mode and
+ *  a find-any mode.  In either case, the elements of the range must be
+ *  monotone (or sorted) with respect to the block.
+ *
+ *  In find-minimum mode (this is a good choice for typical use case),
+ *  the block must return true or false, and there must be a value x
+ *  so that:
+ *
+ *  - the block returns false for any value which is less than x, and
+ *  - the block returns true for any value which is greater than or
+ *    equal to i.
+ *
+ *  If x is within the range, this method returns the value x.
+ *  Otherwise, it returns nil.
+ *
+ *     ary = [0, 4, 7, 10, 12]
+ *     (0...ary.size).bsearch {|i| ary[i] >= 4 } #=> 1
+ *     (0...ary.size).bsearch {|i| ary[i] >= 6 } #=> 2
+ *     (0...ary.size).bsearch {|i| ary[i] >= 8 } #=> 3
+ *     (0...ary.size).bsearch {|i| ary[i] >= 100 } #=> nil
+ *
+ *     (0.0...Float::INFINITY).bsearch {|x| Math.log(x) >= 0 } #=> 1.0
+ *
+ *  In find-any mode (this behaves like libc's bsearch(3)), the block
+ *  must return a number, and there must be two values x and y (x <= y)
+ *  so that:
+ *
+ *  - the block returns a positive number for v if v < x,
+ *  - the block returns zero for v if x <= v < y, and
+ *  - the block returns a negative number for v if y <= v.
+ *
+ *  This method returns any value which is within the intersection of
+ *  the given range and x...y (if any).  If there is no value that
+ *  satisfies the condition, it returns nil.
+ *
+ *     ary = [0, 100, 100, 100, 200]
+ *     (0..4).bsearch {|i| 100 - ary[i] } #=> 1, 2 or 3
+ *     (0..4).bsearch {|i| 300 - ary[i] } #=> nil
+ *     (0..4).bsearch {|i|  50 - ary[i] } #=> nil
+ *
+ *  You must not mix the two modes at a time; the block must always
+ *  return either true/false, or always return a number.  It is
+ *  undefined which value is actually picked up at each iteration.
+ */
+
+static VALUE
+range_bsearch(VALUE range)
+{
+    VALUE beg, end;
+    int smaller, satisfied = 0;
+
+    /* Implementation notes:
+     * Floats are handled by mapping them to 64 bits integers.
+     * Apart from sign issues, floats and their 64 bits integer have the
+     * same order, assuming they are represented as exponent followed
+     * by the mantissa. This is true with or without implicit bit.
+     *
+     * Finding the average of two ints needs to be careful about
+     * potential overflow (since float to long can use 64 bits)
+     * as well as the fact that -1/2 can be 0 or -1 in C89.
+     *
+     * Note that -0.0 is mapped to the same int as 0.0 as we don't want
+     * (-1...0.0).bsearch to yield -0.0.
+     */
+
+#define BSEARCH_CHECK(val) \
+    do { \
+	VALUE v = rb_yield(val); \
+	if (FIXNUM_P(v)) { \
+	    if (FIX2INT(v) == 0) return val; \
+	    smaller = FIX2INT(v) < 0; \
+	} \
+	else if (v == Qtrue) { \
+	    satisfied = 1; \
+	    smaller = 1; \
+	} \
+	else if (v == Qfalse || v == Qnil) { \
+	    smaller = 0; \
+	} \
+	else if (rb_obj_is_kind_of(v, rb_cNumeric)) { \
+	    int cmp = rb_cmpint(rb_funcall(v, id_cmp, 1, INT2FIX(0)), v, INT2FIX(0)); \
+	    if (!cmp) return val; \
+	    smaller = cmp < 0; \
+	} \
+	else { \
+	    rb_raise(rb_eTypeError, "wrong argument type %s" \
+		" (must be numeric, true, false or nil)", \
+		rb_obj_classname(v)); \
+	} \
+    } while (0)
+
+#define BSEARCH(conv) \
+    do { \
+	RETURN_ENUMERATOR(range, 0, 0); \
+	if (EXCL(range)) high--; \
+	org_high = high; \
+	while (low < high) { \
+	    mid = ((high < 0) == (low < 0)) ? low + ((high - low) / 2) \
+		: (low < -high) ? -((-1 - low - high)/2 + 1) : (low + high) / 2; \
+	    BSEARCH_CHECK(conv(mid)); \
+	    if (smaller) { \
+		high = mid; \
+	    } \
+	    else { \
+		low = mid + 1; \
+	    } \
+	} \
+	if (low == org_high) { \
+	    BSEARCH_CHECK(conv(low)); \
+	    if (!smaller) return Qnil; \
+	} \
+	if (!satisfied) return Qnil; \
+	return conv(low); \
+    } while (0)
+
+
+    beg = RANGE_BEG(range);
+    end = RANGE_END(range);
+
+    if (FIXNUM_P(beg) && FIXNUM_P(end)) {
+	long low = FIX2LONG(beg);
+	long high = FIX2LONG(end);
+	long mid, org_high;
+	BSEARCH(INT2FIX);
+    }
+#if SIZEOF_DOUBLE == 8 && defined(HAVE_INT64_T)
+    else if (RB_TYPE_P(beg, T_FLOAT) || RB_TYPE_P(end, T_FLOAT)) {
+	int64_t low  = double_as_int64(RFLOAT_VALUE(rb_Float(beg)));
+	int64_t high = double_as_int64(RFLOAT_VALUE(rb_Float(end)));
+	int64_t mid, org_high;
+	BSEARCH(int64_as_double_to_num);
+    }
+#endif
+    else if (is_integer_p(beg) && is_integer_p(end)) {
+	VALUE low = rb_to_int(beg);
+	VALUE high = rb_to_int(end);
+	VALUE mid, org_high;
+	RETURN_ENUMERATOR(range, 0, 0);
+	if (EXCL(range)) high = rb_funcall(high, '-', 1, INT2FIX(1));
+	org_high = high;
+
+	while (rb_cmpint(rb_funcall(low, id_cmp, 1, high), low, high) < 0) {
+	    mid = rb_funcall(rb_funcall(high, '+', 1, low), id_div, 1, INT2FIX(2));
+	    BSEARCH_CHECK(mid);
+	    if (smaller) {
+		high = mid;
+	    }
+	    else {
+		low = rb_funcall(mid, '+', 1, INT2FIX(1));
+	    }
+	}
+	if (rb_equal(low, org_high)) {
+	    BSEARCH_CHECK(low);
+	    if (!smaller) return Qnil;
+	}
+	if (!satisfied) return Qnil;
+	return low;
+    }
+    else {
+	rb_raise(rb_eTypeError, "can't do binary search for %s", rb_obj_classname(beg));
+    }
+    return range;
+}
+
 static VALUE
 each_i(VALUE v, void *arg)
 {
@@ -398,22 +690,57 @@ each_i(VALUE v, void *arg)
     return Qnil;
 }
 
+static VALUE
+sym_each_i(VALUE v, void *arg)
+{
+    rb_yield(rb_str_intern(v));
+    return Qnil;
+}
+
 /*
  *  call-seq:
- *     rng.each {| i | block } => rng
+ *     rng.size                   -> num
  *
- *  Iterates over the elements <i>rng</i>, passing each in turn to the
- *  block. You can only iterate if the start object of the range
- *  supports the +succ+ method (which means that you can't iterate over
- *  ranges of +Float+ objects).
+ *  Returns the number of elements in the range.
  *
- *     (10..15).each do |n|
- *        print n, ' '
- *     end
+ *    (10..20).size    #=> 11
+ */
+
+static VALUE
+range_size(VALUE range)
+{
+    VALUE b = RANGE_BEG(range), e = RANGE_END(range);
+    if (rb_obj_is_kind_of(b, rb_cNumeric) && rb_obj_is_kind_of(e, rb_cNumeric)) {
+	return ruby_num_interval_step_size(b, e, INT2FIX(1), EXCL(range));
+    }
+    return Qnil;
+}
+
+static VALUE
+range_enum_size(VALUE range, VALUE args, VALUE eobj)
+{
+    return range_size(range);
+}
+
+/*
+ *  call-seq:
+ *     rng.each {| i | block } -> rng
+ *     rng.each                -> an_enumerator
  *
- *  <em>produces:</em>
+ *  Iterates over the elements of range, passing each in turn to the
+ *  block.
  *
- *     10 11 12 13 14 15
+ *  The +each+ method can only be used if the begin object of the range
+ *  supports the +succ+ method.  A TypeError is raised if the object
+ *  does not have +succ+ method defined (like Float).
+ *
+ *  If no block is given, an enumerator is returned instead.
+ *
+ *     (10..15).each {|n| print n, ' ' }
+ *     # prints: 10 11 12 13 14 15
+ *
+ *     (2.5..5).each {|n| print n, ' ' }
+ *     # raises: TypeError: can't iterate from Float
  */
 
 static VALUE
@@ -421,7 +748,7 @@ range_each(VALUE range)
 {
     VALUE beg, end;
 
-    RETURN_ENUMERATOR(range, 0, 0);
+    RETURN_SIZED_ENUMERATOR(range, 0, 0, range_enum_size);
 
     beg = RANGE_BEG(range);
     end = RANGE_END(range);
@@ -436,28 +763,41 @@ range_each(VALUE range)
 	    rb_yield(LONG2FIX(i));
 	}
     }
-    else if (TYPE(beg) == T_STRING) {
+    else if (SYMBOL_P(beg) && SYMBOL_P(end)) { /* symbols are special */
 	VALUE args[2];
 
-	args[0] = end;
+	args[0] = rb_sym_to_s(end);
 	args[1] = EXCL(range) ? Qtrue : Qfalse;
-	rb_block_call(beg, rb_intern("upto"), 2, args, rb_yield, 0);
+	rb_block_call(rb_sym_to_s(beg), rb_intern("upto"), 2, args, sym_each_i, 0);
     }
     else {
-	if (!rb_respond_to(beg, id_succ)) {
-	    rb_raise(rb_eTypeError, "can't iterate from %s",
-		     rb_obj_classname(beg));
+	VALUE tmp = rb_check_string_type(beg);
+
+	if (!NIL_P(tmp)) {
+	    VALUE args[2];
+
+	    args[0] = end;
+	    args[1] = EXCL(range) ? Qtrue : Qfalse;
+	    rb_block_call(tmp, rb_intern("upto"), 2, args, rb_yield, 0);
+	}
+	else {
+	    if (!discrete_object_p(beg)) {
+		rb_raise(rb_eTypeError, "can't iterate from %s",
+			 rb_obj_classname(beg));
+	    }
+	    range_each_func(range, each_i, NULL);
 	}
-	range_each_func(range, each_i, NULL);
     }
     return range;
 }
 
 /*
  *  call-seq:
- *     rng.begin    => obj
+ *     rng.begin    -> obj
  *
- *  Returns the first object in <i>rng</i>.
+ *  Returns the object that defines the beginning of the range.
+ *
+ *      (1..10).begin   #=> 1
  */
 
 static VALUE
@@ -469,9 +809,9 @@ range_begin(VALUE range)
 
 /*
  *  call-seq:
- *     rng.end    => obj
+ *     rng.end    -> obj
  *
- *  Returns the object that defines the end of <i>rng</i>.
+ *  Returns the object that defines the end of the range.
  *
  *     (1..10).end    #=> 10
  *     (1...10).end   #=> 10
@@ -501,10 +841,14 @@ first_i(VALUE i, VALUE *ary)
 
 /*
  *  call-seq:
- *     rng.first    => obj
- *     rng.first(n) => an_array
+ *     rng.first    -> obj
+ *     rng.first(n) -> an_array
  *
- *  Returns the first object in <i>rng</i>, or the first +n+ elements.
+ *  Returns the first object in the range, or an array of the first +n+
+ *  elements.
+ *
+ *    (10..20).first     #=> 10
+ *    (10..20).first(3)  #=> [10, 11, 12]
  */
 
 static VALUE
@@ -517,7 +861,7 @@ range_first(int argc, VALUE *argv, VALUE range)
     rb_scan_args(argc, argv, "1", &n);
     ary[0] = n;
     ary[1] = rb_ary_new2(NUM2LONG(n));
-    rb_block_call(range, rb_intern("each"), 0, 0, first_i, (VALUE)ary);
+    rb_block_call(range, idEach, 0, 0, first_i, (VALUE)ary);
 
     return ary[1];
 }
@@ -525,17 +869,24 @@ range_first(int argc, VALUE *argv, VALUE range)
 
 /*
  *  call-seq:
- *     rng.last    => obj
- *     rng.last(n) => an_array
+ *     rng.last    -> obj
+ *     rng.last(n) -> an_array
  *
- *  Returns the last object in <i>rng</i>, or the last +n+ elements.
+ *  Returns the last object in the range,
+ *  or an array of the last +n+ elements.
+ *
+ *  Note that with no arguments +last+ will return the object that defines
+ *  the end of the range even if #exclude_end? is +true+.
+ *
+ *    (10..20).last      #=> 20
+ *    (10...20).last     #=> 20
+ *    (10..20).last(3)   #=> [18, 19, 20]
+ *    (10...20).last(3)  #=> [17, 18, 19]
  */
 
 static VALUE
 range_last(int argc, VALUE *argv, VALUE range)
 {
-    VALUE rb_ary_last(int, VALUE *, VALUE);
-
     if (argc == 0) return RANGE_END(range);
     return rb_ary_last(argc, argv, rb_Array(range));
 }
@@ -543,13 +894,16 @@ range_last(int argc, VALUE *argv, VALUE range)
 
 /*
  *  call-seq:
- *     rng.min                    => obj
- *     rng.min {| a,b | block }   => obj
+ *     rng.min                    -> obj
+ *     rng.min {| a,b | block }   -> obj
  *
- *  Returns the minimum value in <i>rng</i>. The second uses
- *  the block to compare values.  Returns nil if the first
- *  value in range is larger than the last value.
+ *  Returns the minimum value in the range. Returns +nil+ if the begin
+ *  value of the range is larger than the end value.
  *
+ *  Can be given an optional block to override the default comparison
+ *  method <code>a <=> b</code>.
+ *
+ *    (10..20).min    #=> 10
  */
 
 
@@ -572,13 +926,16 @@ range_min(VALUE range)
 
 /*
  *  call-seq:
- *     rng.max                    => obj
- *     rng.max {| a,b | block }   => obj
+ *     rng.max                    -> obj
+ *     rng.max {| a,b | block }   -> obj
  *
- *  Returns the maximum value in <i>rng</i>. The second uses
- *  the block to compare values.  Returns nil if the first
- *  value in range is larger than the last value.
+ *  Returns the maximum value in the range. Returns +nil+ if the begin
+ *  value of the range larger than the end value.
  *
+ *  Can be given an optional block to override the default comparison
+ *  method <code>a <=> b</code>.
+ *
+ *    (10..20).max    #=> 20
  */
 
 static VALUE
@@ -601,6 +958,9 @@ range_max(VALUE range)
 		rb_raise(rb_eTypeError, "cannot exclude non Integer end value");
 	    }
 	    if (c == 0) return Qnil;
+	    if (!FIXNUM_P(b) && !rb_obj_is_kind_of(b,rb_cInteger)) {
+		rb_raise(rb_eTypeError, "cannot exclude end value with non Integer begin value");
+	    }
 	    if (FIXNUM_P(e)) {
 		return LONG2NUM(FIX2LONG(e) - 1);
 	    }
@@ -622,8 +982,8 @@ rb_range_values(VALUE range, VALUE *begp, VALUE *endp, int *exclp)
 	excl = EXCL(range);
     }
     else {
-	if (!rb_respond_to(range, id_beg)) return Qfalse;
-	if (!rb_respond_to(range, id_end)) return Qfalse;
+	if (!rb_respond_to(range, id_beg)) return (int)Qfalse;
+	if (!rb_respond_to(range, id_end)) return (int)Qfalse;
 	b = rb_funcall(range, id_beg, 0);
 	e = rb_funcall(range, id_end, 0);
 	excl = RTEST(rb_funcall(range, rb_intern("exclude_end?"), 0));
@@ -631,7 +991,7 @@ rb_range_values(VALUE range, VALUE *begp, VALUE *endp, int *exclp)
     *begp = b;
     *endp = e;
     *exclp = excl;
-    return Qtrue;
+    return (int)Qtrue;
 }
 
 VALUE
@@ -652,16 +1012,16 @@ rb_range_beg_len(VALUE range, long *begp, long *lenp, long len, int err)
 	if (beg < 0)
 	    goto out_of_range;
     }
+    if (end < 0)
+	end += len;
+    if (!excl)
+	end++;			/* include end point */
     if (err == 0 || err == 2) {
 	if (beg > len)
 	    goto out_of_range;
 	if (end > len)
 	    end = len;
     }
-    if (end < 0)
-	end += len;
-    if (!excl)
-	end++;			/* include end point */
     len = end - beg;
     if (len < 0)
 	len = 0;
@@ -680,9 +1040,10 @@ rb_range_beg_len(VALUE range, long *begp, long *lenp, long len, int err)
 
 /*
  * call-seq:
- *   rng.to_s   => string
+ *   rng.to_s   -> string
  *
- * Convert this range object to a printable form.
+ * Convert this range object to a printable form (using #to_s to convert the
+ * begin and end objects).
  */
 
 static VALUE
@@ -720,10 +1081,10 @@ inspect_range(VALUE range, VALUE dummy, int recur)
 
 /*
  * call-seq:
- *   rng.inspect  => string
+ *   rng.inspect  -> string
  *
  * Convert this range object to a printable form (using
- * <code>inspect</code> to convert the start and end
+ * <code>inspect</code> to convert the begin and end
  * objects).
  */
 
@@ -736,12 +1097,11 @@ range_inspect(VALUE range)
 
 /*
  *  call-seq:
- *     rng === obj       =>  true or false
+ *     rng === obj       ->  true or false
  *
- *  Returns <code>true</code> if <i>obj</i> is an element of
- *  <i>rng</i>, <code>false</code> otherwise. Conveniently,
- *  <code>===</code> is the comparison operator used by
- *  <code>case</code> statements.
+ *  Returns <code>true</code> if +obj+ is an element of the range,
+ *  <code>false</code> otherwise.  Conveniently, <code>===</code> is the
+ *  comparison operator used by <code>case</code> statements.
  *
  *     case 79
  *     when 1..50   then   print "low\n"
@@ -763,15 +1123,16 @@ range_eqq(VALUE range, VALUE val)
 
 /*
  *  call-seq:
- *     rng.member?(val)  =>  true or false
- *     rng.include?(val) =>  true or false
+ *     rng.member?(obj)  ->  true or false
+ *     rng.include?(obj) ->  true or false
  *
- *  Returns <code>true</code> if <i>obj</i> is an element of
- *  <i>rng</i>, <code>false</code> otherwise.  If beg and end are
- *  numeric, comparison is done according magnitude of values.
+ *  Returns <code>true</code> if +obj+ is an element of
+ *  the range, <code>false</code> otherwise.  If begin and end are
+ *  numeric, comparison is done according to the magnitude of the values.
  *
- *     ("a".."z").include?("g")  # => true
- *     ("a".."z").include?("A")  # => false
+ *     ("a".."z").include?("g")   #=> true
+ *     ("a".."z").include?("A")   #=> false
+ *     ("a".."z").include?("cc")  #=> false
  */
 
 static VALUE
@@ -798,10 +1159,10 @@ range_include(VALUE range, VALUE val)
 	}
 	return Qfalse;
     }
-    else if (TYPE(beg) == T_STRING && TYPE(end) == T_STRING &&
+    else if (RB_TYPE_P(beg, T_STRING) && RB_TYPE_P(end, T_STRING) &&
 	     RSTRING_LEN(beg) == 1 && RSTRING_LEN(end) == 1) {
 	if (NIL_P(val)) return Qfalse;
-	if (TYPE(val) == T_STRING) {
+	if (RB_TYPE_P(val, T_STRING)) {
 	    if (RSTRING_LEN(val) == 0 || RSTRING_LEN(val) > 1)
 		return Qfalse;
 	    else {
@@ -824,14 +1185,17 @@ range_include(VALUE range, VALUE val)
 
 /*
  *  call-seq:
- *     rng.cover?(val)  =>  true or false
+ *     rng.cover?(obj)  ->  true or false
  *
- *  Returns <code>true</code> if <i>obj</i> is between beg and end,
- *  i.e <code>beg <= obj <= end</code> (or <i>end</i> exclusive when
- *  <code>exclude_end?</code> is true).
+ *  Returns <code>true</code> if +obj+ is between the begin and end of
+ *  the range.
+ *
+ *  This tests <code>begin <= obj <= end</code> when #exclude_end? is +false+
+ *  and <code>begin <= obj < end</code> when #exclude_end? is +true+.
  *
  *     ("a".."z").cover?("c")    #=> true
  *     ("a".."z").cover?("5")    #=> false
+ *     ("a".."z").cover?("cc")   #=> true
  */
 
 static VALUE
@@ -858,8 +1222,7 @@ static VALUE
 range_dumper(VALUE range)
 {
     VALUE v;
-    NEWOBJ(m, struct RObject);
-    OBJSETUP(m, rb_cObject, T_OBJECT);
+    NEWOBJ_OF(m, struct RObject, rb_cObject, T_OBJECT | (RGENGC_WB_PROTECTED_OBJECT ? FL_WB_PROTECTED : 1));
 
     v = (VALUE)m;
 
@@ -872,13 +1235,13 @@ range_dumper(VALUE range)
 static VALUE
 range_loader(VALUE range, VALUE obj)
 {
-    if (TYPE(obj) != T_OBJECT || RBASIC(obj)->klass != rb_cObject) {
+    if (!RB_TYPE_P(obj, T_OBJECT) || RBASIC(obj)->klass != rb_cObject) {
         rb_raise(rb_eTypeError, "not a dumped range object");
     }
 
-    RSTRUCT(range)->as.ary[0] = rb_ivar_get(obj, id_beg);
-    RSTRUCT(range)->as.ary[1] = rb_ivar_get(obj, id_end);
-    RSTRUCT(range)->as.ary[2] = rb_ivar_get(obj, id_excl);
+    RSTRUCT_SET(range, 0, rb_ivar_get(obj, id_beg));
+    RSTRUCT_SET(range, 1, rb_ivar_get(obj, id_end));
+    RSTRUCT_SET(range, 2, rb_ivar_get(obj, id_excl));
     return range;
 }
 
@@ -886,16 +1249,16 @@ static VALUE
 range_alloc(VALUE klass)
 {
   /* rb_struct_alloc_noinit itself should not be used because
-   * rb_marshal_define_compat uses equality of allocaiton function */
+   * rb_marshal_define_compat uses equality of allocation function */
     return rb_struct_alloc_noinit(klass);
 }
 
 /*  A <code>Range</code> represents an interval---a set of values with a
- *  start and an end. Ranges may be constructed using the
+ *  beginning and an end. Ranges may be constructed using the
  *  <em>s</em><code>..</code><em>e</em> and
  *  <em>s</em><code>...</code><em>e</em> literals, or with
- *  <code>Range::new</code>. Ranges constructed using <code>..</code>
- *  run from the start to the end inclusively. Those created using
+ *  Range::new. Ranges constructed using <code>..</code>
+ *  run from the beginning to the end inclusively. Those created using
  *  <code>...</code> exclude the end value. When used as an iterator,
  *  ranges return each value in the sequence.
  *
@@ -904,10 +1267,20 @@ range_alloc(VALUE klass)
  *     ('a'..'e').to_a    #=> ["a", "b", "c", "d", "e"]
  *     ('a'...'e').to_a   #=> ["a", "b", "c", "d"]
  *
- *  Ranges can be constructed using objects of any type, as long as the
- *  objects can be compared using their <code><=></code> operator and
- *  they support the <code>succ</code> method to return the next object
- *  in sequence.
+ *  == Custom Objects in Ranges
+ *
+ *  Ranges can be constructed using any objects that can be compared
+ *  using the <code><=></code> operator.
+ *  Methods that treat the range as a sequence (#each and methods inherited
+ *  from Enumerable) expect the begin object to implement a
+ *  <code>succ</code> method to return the next object in sequence.
+ *  The #step and #include? methods require the begin
+ *  object to implement <code>succ</code> or to be numeric.
+ *
+ *  In the <code>Xs</code> class below both <code><=></code> and
+ *  <code>succ</code> are implemented so <code>Xs</code> can be used
+ *  to construct ranges. Note that the Comparable module is included
+ *  so the <code>==</code> method is defined in terms of <code><=></code>.
  *
  *     class Xs                # represent a string of 'x's
  *       include Comparable
@@ -929,17 +1302,12 @@ range_alloc(VALUE klass)
  *       end
  *     end
  *
+ *  An example of using <code>Xs</code> to construct a range:
+ *
  *     r = Xs.new(3)..Xs.new(6)   #=> xxx..xxxxxx
  *     r.to_a                     #=> [xxx, xxxx, xxxxx, xxxxxx]
  *     r.member?(Xs.new(5))       #=> true
  *
- *  In the previous code example, class <code>Xs</code> includes the
- *  <code>Comparable</code> module. This is because
- *  <code>Enumerable#member?</code> checks for equality using
- *  <code>==</code>. Including <code>Comparable</code> ensures that the
- *  <code>==</code> method is defined in terms of the <code><=></code>
- *  method implemented in <code>Xs</code>.
- *
  */
 
 void
@@ -953,6 +1321,8 @@ Init_Range(void)
     id_beg = rb_intern("begin");
     id_end = rb_intern("end");
     id_excl = rb_intern("excl");
+    id_integer_p = rb_intern("integer?");
+    id_div = rb_intern("div");
 
     rb_cRange = rb_struct_define_without_accessor(
         "Range", rb_cObject, range_alloc,
@@ -968,12 +1338,14 @@ Init_Range(void)
     rb_define_method(rb_cRange, "hash", range_hash, 0);
     rb_define_method(rb_cRange, "each", range_each, 0);
     rb_define_method(rb_cRange, "step", range_step, -1);
+    rb_define_method(rb_cRange, "bsearch", range_bsearch, 0);
     rb_define_method(rb_cRange, "begin", range_begin, 0);
     rb_define_method(rb_cRange, "end", range_end, 0);
     rb_define_method(rb_cRange, "first", range_first, -1);
     rb_define_method(rb_cRange, "last", range_last, -1);
     rb_define_method(rb_cRange, "min", range_min, 0);
     rb_define_method(rb_cRange, "max", range_max, 0);
+    rb_define_method(rb_cRange, "size", range_size, 0);
     rb_define_method(rb_cRange, "to_s", range_to_s, 0);
     rb_define_method(rb_cRange, "inspect", range_inspect, 0);
 
