@@ -1,6 +1,5 @@
 require 'test/unit'
-require 'tmpdir'
-require 'pathname'
+require 'tempfile'
 require 'timeout'
 require_relative 'envutil'
 require 'rbconfig'
@@ -16,6 +15,13 @@ class TestProcess < Test::Unit::TestCase
     Process.waitall
   end
 
+  def windows?
+    self.class.windows?
+  end
+  def self.windows?
+    return /mswin|mingw|bccwin/ =~ RUBY_PLATFORM
+  end
+
   def write_file(filename, content)
     File.open(filename, "w") {|f|
       f << content
@@ -24,7 +30,7 @@ class TestProcess < Test::Unit::TestCase
 
   def with_tmpchdir
     Dir.mktmpdir {|d|
-      d = Pathname.new(d).realpath.to_s
+      d = File.realpath(d)
       Dir.chdir(d) {
         yield d
       }
@@ -59,9 +65,11 @@ class TestProcess < Test::Unit::TestCase
     return unless rlimit_exist?
     with_tmpchdir {
       write_file 's', <<-"End"
+	# if limit=0, this test freeze pn OpenBSD
+	limit = /openbsd/ =~ RUBY_PLATFORM ? 1 : 0
 	result = 1
 	begin
-	  Process.setrlimit(Process::RLIMIT_NOFILE, 0)
+	  Process.setrlimit(Process::RLIMIT_NOFILE, limit)
 	rescue Errno::EINVAL
 	  result = 0
 	end
@@ -152,7 +160,7 @@ class TestProcess < Test::Unit::TestCase
   end
 
   def test_execopts_pgroup
-    skip "system(:pgroup) is not supported" if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+    skip "system(:pgroup) is not supported" if windows?
     assert_nothing_raised { system(*TRUECOMMAND, :pgroup=>false) }
 
     io = IO.popen([RUBY, "-e", "print Process.getpgrp"])
@@ -229,6 +237,8 @@ class TestProcess < Test::Unit::TestCase
     MANDATORY_ENVS << 'LD_PRELOAD'
   when /mswin|mingw/
     MANDATORY_ENVS.concat(%w[HOME USER TMPDIR])
+  when /darwin/
+    MANDATORY_ENVS.concat(ENV.keys.grep(/\A__CF_/))
   end
   if e = RbConfig::CONFIG['LIBPATHENV']
     MANDATORY_ENVS << e
@@ -297,6 +307,78 @@ class TestProcess < Test::Unit::TestCase
     end
   end
 
+  def _test_execopts_env_popen(cmd)
+    message = cmd.inspect
+    IO.popen({"FOO"=>"BAR"}, cmd) {|io|
+      assert_equal('FOO=BAR', io.read[/^FOO=.*/], message)
+    }
+
+    old = ENV["hmm"]
+    begin
+      ENV["hmm"] = "fufu"
+      IO.popen(cmd) {|io| assert_match(/^hmm=fufu$/, io.read, message)}
+      IO.popen({"hmm"=>""}, cmd) {|io| assert_match(/^hmm=$/, io.read, message)}
+      IO.popen({"hmm"=>nil}, cmd) {|io| assert_not_match(/^hmm=/, io.read, message)}
+      ENV["hmm"] = ""
+      IO.popen(cmd) {|io| assert_match(/^hmm=$/, io.read, message)}
+      IO.popen({"hmm"=>""}, cmd) {|io| assert_match(/^hmm=$/, io.read, message)}
+      IO.popen({"hmm"=>nil}, cmd) {|io| assert_not_match(/^hmm=/, io.read, message)}
+      ENV["hmm"] = nil
+      IO.popen(cmd) {|io| assert_not_match(/^hmm=/, io.read, message)}
+      IO.popen({"hmm"=>""}, cmd) {|io| assert_match(/^hmm=$/, io.read, message)}
+      IO.popen({"hmm"=>nil}, cmd) {|io| assert_not_match(/^hmm=/, io.read, message)}
+    ensure
+      ENV["hmm"] = old
+    end
+  end
+
+  def test_execopts_env_popen_vector
+    _test_execopts_env_popen(ENVCOMMAND)
+  end
+
+  def test_execopts_env_popen_string
+    with_tmpchdir do |d|
+      open('test-script', 'w') do |f|
+        ENVCOMMAND.each_with_index do |cmd, i|
+          next if i.zero? or cmd == "-e"
+          f.puts cmd
+        end
+      end
+      _test_execopts_env_popen("#{RUBY} test-script")
+    end
+  end
+
+  def test_execopts_preserve_env_on_exec_failure
+    with_tmpchdir {|d|
+      write_file 's', <<-"End"
+        ENV["mgg"] = nil
+        prog = "./nonexistent"
+        begin
+          Process.exec({"mgg" => "mggoo"}, [prog, prog])
+        rescue Errno::ENOENT
+        end
+        open('out', 'w') {|f|
+          f.print ENV["mgg"].inspect
+        }
+      End
+      system(RUBY, 's')
+      assert_equal(nil.inspect, File.read('out'),
+        "[ruby-core:44093] [ruby-trunk - Bug #6249]")
+    }
+  end
+
+  def test_execopts_env_single_word
+    with_tmpchdir {|d|
+      open("test_execopts_env_single_word.rb", "w") {|f|
+        f.puts "print ENV['hgga']"
+      }
+      system({"hgga"=>"ugu"}, RUBY,
+             :in => 'test_execopts_env_single_word.rb',
+             :out => 'test_execopts_env_single_word.out')
+      assert_equal('ugu', File.read('test_execopts_env_single_word.out'))
+    }
+  end
+
   def test_execopts_unsetenv_others
     h = {}
     MANDATORY_ENVS.each {|k| e = ENV[k] and h[k] = e}
@@ -321,10 +403,20 @@ class TestProcess < Test::Unit::TestCase
     }
   end
 
+  def test_execopts_open_chdir
+    with_tmpchdir {|d|
+      Dir.mkdir "foo"
+      system(*PWD, :chdir => "foo", :out => "open_chdir_test")
+      assert_file.exist?("open_chdir_test")
+      assert_file.not_exist?("foo/open_chdir_test")
+      assert_equal("#{d}/foo", File.read("open_chdir_test").chomp)
+    }
+  end
+
   UMASK = [RUBY, '-e', 'printf "%04o\n", File.umask']
 
   def test_execopts_umask
-    skip "umask is not supported" if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+    skip "umask is not supported" if windows?
     IO.popen([*UMASK, :umask => 0]) {|io|
       assert_equal("0000", io.read.chomp)
     }
@@ -366,7 +458,7 @@ class TestProcess < Test::Unit::TestCase
     with_tmpchdir {|d|
       Process.wait Process.spawn(*ECHO["a"], STDOUT=>["out", File::WRONLY|File::CREAT|File::TRUNC, 0644])
       assert_equal("a", File.read("out").chomp)
-      if /mswin|mingw/ =~ RUBY_PLATFORM
+      if windows?
         # currently telling to child the file modes is not supported.
         open("out", "a") {|f| f.write "0\n"}
       else
@@ -389,18 +481,14 @@ class TestProcess < Test::Unit::TestCase
         assert_equal("d", File.read("out").chomp)
       }
       opts = {STDOUT=>["out", File::WRONLY|File::CREAT|File::TRUNC, 0644]}
-      if /mswin|mingw/ !~ RUBY_PLATFORM
-        opts.merge(3=>STDOUT, 4=>STDOUT, 5=>STDOUT, 6=>STDOUT, 7=>STDOUT)
-      end
+      opts.merge(3=>STDOUT, 4=>STDOUT, 5=>STDOUT, 6=>STDOUT, 7=>STDOUT) unless windows?
       Process.wait Process.spawn(*ECHO["e"], opts)
       assert_equal("e", File.read("out").chomp)
       opts = {STDOUT=>["out", File::WRONLY|File::CREAT|File::TRUNC, 0644]}
-      if /mswin|mingw/ !~ RUBY_PLATFORM
-        opts.merge(3=>0, 4=>:in, 5=>STDIN, 6=>1, 7=>:out, 8=>STDOUT, 9=>2, 10=>:err, 11=>STDERR)
-      end
+      opts.merge(3=>0, 4=>:in, 5=>STDIN, 6=>1, 7=>:out, 8=>STDOUT, 9=>2, 10=>:err, 11=>STDERR) unless windows?
       Process.wait Process.spawn(*ECHO["ee"], opts)
       assert_equal("ee", File.read("out").chomp)
-      if /mswin|mingw/ !~ RUBY_PLATFORM
+      unless windows?
         # passing non-stdio fds is not supported on Windows
         File.open("out", "w") {|f|
           h = {STDOUT=>f, f=>STDOUT}
@@ -426,7 +514,7 @@ class TestProcess < Test::Unit::TestCase
       Process.wait Process.spawn(*SORT, STDIN=>"out", STDOUT=>"out2")
       assert_equal("ggg\nhhh\n", File.read("out2"))
 
-      if /mswin|mingw/ !~ RUBY_PLATFORM
+      unless windows?
         # passing non-stdio fds is not supported on Windows
         assert_raise(Errno::ENOENT) {
           Process.wait Process.spawn("non-existing-command", (3..60).to_a=>["err", File::WRONLY|File::CREAT])
@@ -441,7 +529,9 @@ class TestProcess < Test::Unit::TestCase
 
       with_pipe {|r1, w1|
         with_pipe {|r2, w2|
-          pid = spawn(*SORT, STDIN=>r1, STDOUT=>w2, w1=>:close, r2=>:close)
+          opts = {STDIN=>r1, STDOUT=>w2}
+          opts.merge(w1=>:close, r2=>:close) unless windows?
+          pid = spawn(*SORT, opts)
           r1.close
           w2.close
           w1.puts "c"
@@ -454,15 +544,15 @@ class TestProcess < Test::Unit::TestCase
         }
       }
 
-      if /mswin|mingw/ !~ RUBY_PLATFORM
+      unless windows?
         # passing non-stdio fds is not supported on Windows
         with_pipes(5) {|pipes|
           ios = pipes.flatten
           h = {}
           ios.length.times {|i| h[ios[i]] = ios[(i-1)%ios.length] }
           h2 = h.invert
-          rios = pipes.map {|r, w| r }
-          wios = pipes.map {|r, w| w }
+          _rios = pipes.map {|r, w| r }
+          wios  = pipes.map {|r, w| w }
           child_wfds = wios.map {|w| h2[w].fileno }
           pid = spawn(RUBY, "-e",
                   "[#{child_wfds.join(',')}].each {|fd| IO.new(fd, 'w').puts fd }", h)
@@ -477,8 +567,8 @@ class TestProcess < Test::Unit::TestCase
           h = {}
           ios.length.times {|i| h[ios[i]] = ios[(i+1)%ios.length] }
           h2 = h.invert
-          rios = pipes.map {|r, w| r }
-          wios = pipes.map {|r, w| w }
+          _rios = pipes.map {|r, w| r }
+          wios  = pipes.map {|r, w| w }
           child_wfds = wios.map {|w| h2[w].fileno }
           pid = spawn(RUBY, "-e",
                   "[#{child_wfds.join(',')}].each {|fd| IO.new(fd, 'w').puts fd }", h)
@@ -526,9 +616,7 @@ class TestProcess < Test::Unit::TestCase
                          STDERR=>"out", STDOUT=>[:child, STDERR])
       assert_equal("errout", File.read("out"))
 
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
-        skip "inheritance of fd other than stdin,stdout and stderr is not supported"
-      end
+      skip "inheritance of fd other than stdin,stdout and stderr is not supported" if windows?
       Process.wait spawn(RUBY, "-e", "STDERR.print 'err'; STDOUT.print 'out'",
                          STDOUT=>"out",
                          STDERR=>[:child, 3],
@@ -571,9 +659,7 @@ class TestProcess < Test::Unit::TestCase
       assert_raise(ArgumentError) {
         IO.popen([*ECHO["fuga"], STDOUT=>"out"]) {|io| }
       }
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
-        skip "inheritance of fd other than stdin,stdout and stderr is not supported"
-      end
+      skip "inheritance of fd other than stdin,stdout and stderr is not supported" if windows?
       with_pipe {|r, w|
         IO.popen([RUBY, '-e', 'IO.new(3, "w").puts("a"); puts "b"', 3=>w]) {|io|
           assert_equal("b\n", io.read)
@@ -590,7 +676,6 @@ class TestProcess < Test::Unit::TestCase
   end
 
   def test_popen_fork
-    return if /freebsd/ =~ RUBY_PLATFORM # this test freeze in FreeBSD
     IO.popen("-") {|io|
       if !io
         puts "fooo"
@@ -602,11 +687,9 @@ class TestProcess < Test::Unit::TestCase
   end
 
   def test_fd_inheritance
-    if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
-      skip "inheritance of fd other than stdin,stdout and stderr is not supported"
-    end
+    skip "inheritance of fd other than stdin,stdout and stderr is not supported" if windows?
     with_pipe {|r, w|
-      system(RUBY, '-e', 'IO.new(ARGV[0].to_i, "w").puts(:ba)', w.fileno.to_s)
+      system(RUBY, '-e', 'IO.new(ARGV[0].to_i, "w").puts(:ba)', w.fileno.to_s, w=>w)
       w.close
       assert_equal("ba\n", r.read)
     }
@@ -622,8 +705,9 @@ class TestProcess < Test::Unit::TestCase
 	write_file("s", <<-"End")
 	  exec(#{RUBY.dump}, '-e',
 	       'IO.new(ARGV[0].to_i, "w").puts("bu") rescue nil',
-	       #{w.fileno.to_s.dump})
+	       #{w.fileno.to_s.dump}, :close_others=>false)
 	End
+        w.close_on_exec = false
 	Process.wait spawn(RUBY, "s", :close_others=>false)
 	w.close
 	assert_equal("bu\n", r.read)
@@ -646,9 +730,7 @@ class TestProcess < Test::Unit::TestCase
   end
 
   def test_execopts_close_others
-    if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
-      skip "inheritance of fd other than stdin,stdout and stderr is not supported"
-    end
+    skip "inheritance of fd other than stdin,stdout and stderr is not supported" if windows?
     with_tmpchdir {|d|
       with_pipe {|r, w|
         system(RUBY, '-e', 'STDERR.reopen("err", "w"); IO.new(ARGV[0].to_i, "w").puts("ma")', w.fileno.to_s, :close_others=>true)
@@ -665,6 +747,7 @@ class TestProcess < Test::Unit::TestCase
         File.unlink("err")
       }
       with_pipe {|r, w|
+        w.close_on_exec = false
         Process.wait spawn(RUBY, '-e', 'IO.new(ARGV[0].to_i, "w").puts("bi")', w.fileno.to_s, :close_others=>false)
         w.close
         assert_equal("bi\n", r.read)
@@ -691,6 +774,7 @@ class TestProcess < Test::Unit::TestCase
         Process.wait
       }
       with_pipe {|r, w|
+        w.close_on_exec = false
         io = IO.popen([RUBY, "-e", "STDERR.reopen(STDOUT); IO.new(#{w.fileno}, 'w').puts('mo')", :close_others=>false])
         w.close
         errmsg = io.read
@@ -699,6 +783,7 @@ class TestProcess < Test::Unit::TestCase
         Process.wait
       }
       with_pipe {|r, w|
+        w.close_on_exec = false
         io = IO.popen([RUBY, "-e", "STDERR.reopen(STDOUT); IO.new(#{w.fileno}, 'w').puts('mo')", :close_others=>nil])
         w.close
         errmsg = io.read
@@ -725,6 +810,18 @@ class TestProcess < Test::Unit::TestCase
     end
   end
 
+  def test_execopts_redirect_tempfile
+    bug6269 = '[ruby-core:44181]'
+    Tempfile.create("execopts") do |tmp|
+      pid = assert_nothing_raised(ArgumentError, bug6269) do
+        break spawn(RUBY, "-e", "print $$", out: tmp)
+      end
+      Process.wait(pid)
+      tmp.rewind
+      assert_equal(pid.to_s, tmp.read)
+    end
+  end
+
   def test_execopts_duplex_io
     IO.popen("#{RUBY} -e ''", "r+") {|duplex|
       assert_raise(ArgumentError) { system("#{RUBY} -e ''", duplex=>STDOUT) }
@@ -830,6 +927,25 @@ class TestProcess < Test::Unit::TestCase
     }
   end
 
+  def test_popen_wordsplit_beginning_and_trailing_spaces
+    with_tmpchdir {|d|
+      write_file("script", <<-'End')
+        print "fufumm pid=#{$$} ppid=#{Process.ppid}"
+        exit 7
+      End
+      str = " #{RUBY} script "
+      io = IO.popen(str)
+      pid = io.pid
+      result = io.read
+      io.close
+      status = $?
+      assert_equal(pid, status.pid)
+      assert(status.exited?)
+      assert_equal(7, status.exitstatus)
+      assert_equal("fufumm pid=#{status.pid} ppid=#{$$}", result)
+    }
+  end
+
   def test_exec_wordsplit
     with_tmpchdir {|d|
       write_file("script", <<-'End')
@@ -852,7 +968,7 @@ class TestProcess < Test::Unit::TestCase
       assert_equal(pid, status.pid)
       assert(status.exited?)
       assert_equal(6, status.exitstatus)
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+      if windows?
         expected = "hehe ppid=#{status.pid}"
       else
         expected = "hehe pid=#{status.pid} ppid=#{$$}"
@@ -881,7 +997,7 @@ class TestProcess < Test::Unit::TestCase
       assert_match(/\Ataki pid=\d+ ppid=\d+\z/, result2)
       assert_not_equal(result1[/\d+/].to_i, status.pid)
 
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+      if windows?
         Dir.mkdir(path = "path with space")
         write_file(bat = path + "/bat test.bat", "@echo %1>out")
         system(bat, "foo 'bar'")
@@ -913,7 +1029,7 @@ class TestProcess < Test::Unit::TestCase
       assert_match(/\Atake pid=\d+ ppid=\d+\z/, result2)
       assert_not_equal(result1[/\d+/].to_i, status.pid)
 
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+      if windows?
         Dir.mkdir(path = "path with space")
         write_file(bat = path + "/bat test.bat", "@echo %1>out")
         pid = spawn(bat, "foo 'bar'")
@@ -951,7 +1067,7 @@ class TestProcess < Test::Unit::TestCase
       assert_match(/\Atako pid=\d+ ppid=\d+\ntika pid=\d+ ppid=\d+\n\z/, result)
       assert_not_equal(result[/\d+/].to_i, status.pid)
 
-      if /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+      if windows?
         Dir.mkdir(path = "path with space")
         write_file(bat = path + "/bat test.bat", "@echo %1")
         r = IO.popen([bat, "foo 'bar'"]) {|f| f.read}
@@ -1163,9 +1279,29 @@ class TestProcess < Test::Unit::TestCase
   end
 
   def test_geteuid
+    assert_kind_of(Integer, Process.euid)
+  end
+
+  def test_seteuid
+    assert_nothing_raised(TypeError) {Process.euid += 0}
+  rescue NotImplementedError
+  end
+
+  def test_seteuid_name
+    user = ENV["USER"] or return
+    assert_nothing_raised(TypeError) {Process.euid = user}
+  rescue NotImplementedError
+  end
+
+  def test_getegid
     assert_kind_of(Integer, Process.egid)
   end
 
+  def test_setegid
+    assert_nothing_raised(TypeError) {Process.egid += 0}
+  rescue NotImplementedError
+  end
+
   def test_uid_re_exchangeable_p
     r = Process::UID.re_exchangeable?
     assert(true == r || false == r)
@@ -1218,13 +1354,13 @@ class TestProcess < Test::Unit::TestCase
       Dir.chdir("vd") {
         dir = "#{d}/vd"
         # OpenSolaris cannot remove the current directory.
-        system(RUBY, "--disable-gems", "-e", "Dir.chdir '..'; Dir.rmdir #{dir.dump}")
+        system(RUBY, "--disable-gems", "-e", "Dir.chdir '..'; Dir.rmdir #{dir.dump}", err: File::NULL)
         system({"RUBYLIB"=>nil}, RUBY, "--disable-gems", "-e", "exit true")
         status = $?
       }
       assert(status.success?, "[ruby-dev:38105]")
     }
-  end unless /mswin|bccwin|mingw/ =~ RUBY_PLATFORM
+  end
 
   def test_fallback_to_sh
     feature = '[ruby-core:32745]'
@@ -1237,25 +1373,52 @@ class TestProcess < Test::Unit::TestCase
       open("tmp_script.#{$$}", "w") {|f| f.puts "echo $#: $@"; f.chmod(0755)}
       result = IO.popen(["./tmp_script.#{$$}", "a b", "c"]) {|f| f.read}
       assert_equal("2: a b c\n", result, feature)
+
+      open("tmp_script.#{$$}", "w") {|f| f.puts "echo $hghg"; f.chmod(0755)}
+      result = IO.popen([{"hghg" => "mogomogo"}, "./tmp_script.#{$$}", "a b", "c"]) {|f| f.read}
+      assert_equal("mogomogo\n", result, feature)
+
     end
   end if File.executable?("/bin/sh")
 
-  def test_too_long_path
+  def test_spawn_too_long_path
     bug4314 = '[ruby-core:34842]'
-    exs = [Errno::ENOENT]
-    exs << Errno::E2BIG if defined?(Errno::E2BIG)
-    assert_raise(*exs, bug4314) {Process.spawn("a" * 10_000_000)}
+    assert_fail_too_long_path(%w"echo", bug4314)
   end
 
-  def test_too_long_path2
+  def test_aspawn_too_long_path
     bug4315 = '[ruby-core:34833]'
+    assert_fail_too_long_path(%w"echo |", bug4315)
+  end
+
+  def assert_fail_too_long_path((cmd, sep), mesg)
+    sep ||= ""
+    min = 1_000 / (cmd.size + sep.size)
+    cmds = Array.new(min, cmd)
     exs = [Errno::ENOENT]
     exs << Errno::E2BIG if defined?(Errno::E2BIG)
-    assert_raise(*exs, bug4315) {Process.spawn('"a"|'*10_000_000)}
+    EnvUtil.suppress_warning do
+      assert_raise(*exs, mesg) do
+        begin
+          loop do
+            Process.spawn(cmds.join(sep), [STDOUT, STDERR]=>File::NULL)
+            min = [cmds.size, min].max
+            cmds *= 100
+          end
+        rescue NoMemoryError
+          size = cmds.size
+          raise if min >= size - 1
+          min = [min, size /= 2].max
+          cmds[size..-1] = []
+          raise if size < 250
+          retry
+        end
+      end
+    end
   end
 
   def test_system_sigpipe
-    return if /mswin|mingw/ =~ RUBY_PLATFORM
+    return if windows?
 
     pid = 0
 
@@ -1311,7 +1474,16 @@ class TestProcess < Test::Unit::TestCase
       assert_equal("ok?\n", data)
     end
 
-    if File.directory?("/proc/self/task")
+    def test_daemon_pid
+      cpid, dpid = IO.popen("-", "r+") do |f|
+        break f.pid, Integer(f.read) if f
+        Process.daemon(false, true)
+        puts $$
+      end
+      assert_not_equal(cpid, dpid)
+    end
+
+    if File.directory?("/proc/self/task") && /netbsd[a-z]*[1-6]/ !~ RUBY_PLATFORM
       def test_daemon_no_threads
         pid, data = IO.popen("-", "r+") do |f|
           break f.pid, f.readlines if f
@@ -1322,6 +1494,334 @@ class TestProcess < Test::Unit::TestCase
         assert_equal(2, data.size, bug4920)
         assert_not_include(data.map(&:to_i), pid)
       end
+    else # darwin
+      def test_daemon_no_threads
+        data = Timeout.timeout(3) do
+          IO.popen("-") do |f|
+            break f.readlines.map(&:chomp) if f
+            th = Thread.start {sleep 3}
+            Process.daemon(true, true)
+            puts Thread.list.size, th.status.inspect
+          end
+        end
+        assert_equal(["1", "false"], data)
+      end
     end
   end
+
+  def test_popen_cloexec
+    return unless defined? Fcntl::FD_CLOEXEC
+    IO.popen([RUBY, "-e", ""]) {|io|
+      assert(io.close_on_exec?)
+    }
+  end
+
+  def test_execopts_new_pgroup
+    return unless windows?
+
+    assert_nothing_raised { system(*TRUECOMMAND, :new_pgroup=>true) }
+    assert_nothing_raised { system(*TRUECOMMAND, :new_pgroup=>false) }
+    assert_nothing_raised { spawn(*TRUECOMMAND, :new_pgroup=>true) }
+    assert_nothing_raised { IO.popen([*TRUECOMMAND, :new_pgroup=>true]) {} }
+  end
+
+  def test_execopts_uid
+    feature6975 = '[ruby-core:47414]'
+
+    [30000, [Process.uid, ENV["USER"]]].each do |uid, user|
+      if user
+        assert_nothing_raised(feature6975) do
+          begin
+            system(*TRUECOMMAND, uid: user)
+          rescue Errno::EPERM, NotImplementedError
+          end
+        end
+      end
+
+      assert_nothing_raised(feature6975) do
+        begin
+          system(*TRUECOMMAND, uid: uid)
+        rescue Errno::EPERM, NotImplementedError
+        end
+      end
+
+      assert_nothing_raised(feature6975) do
+        begin
+          u = IO.popen([RUBY, "-e", "print Process.uid", uid: user||uid], &:read)
+          assert_equal(uid.to_s, u, feature6975)
+        rescue Errno::EPERM, NotImplementedError
+        end
+      end
+    end
+  end
+
+  def test_execopts_gid
+    skip "Process.groups not implemented on Windows platform" if windows?
+    feature6975 = '[ruby-core:47414]'
+
+    [30000, *Process.groups.map {|g| g = Etc.getgrgid(g); [g.name, g.gid]}].each do |group, gid|
+      assert_nothing_raised(feature6975) do
+        begin
+          system(*TRUECOMMAND, gid: group)
+        rescue Errno::EPERM, NotImplementedError
+        end
+      end
+
+      gid = "#{gid || group}"
+      assert_nothing_raised(feature6975) do
+        begin
+          g = IO.popen([RUBY, "-e", "print Process.gid", gid: group], &:read)
+          assert_equal(gid, g, feature6975)
+        rescue Errno::EPERM, NotImplementedError
+        end
+      end
+    end
+  end
+
+  def test_sigpipe
+    system(RUBY, "-e", "")
+    with_pipe {|r, w|
+      r.close
+      assert_raise(Errno::EPIPE) { w.print "a" }
+    }
+  end
+
+  def test_sh_comment
+    IO.popen("echo a # fofoof") {|f|
+      assert_equal("a\n", f.read)
+    }
+  end if File.executable?("/bin/sh")
+
+  def test_sh_env
+    IO.popen("foofoo=barbar env") {|f|
+      lines = f.readlines
+      assert_operator(lines, :include?, "foofoo=barbar\n")
+    }
+  end if File.executable?("/bin/sh")
+
+  def test_sh_exec
+    IO.popen("exec echo exexexec") {|f|
+      assert_equal("exexexec\n", f.read)
+    }
+  end if File.executable?("/bin/sh")
+
+  def test_setsid
+    return unless Process.respond_to?(:setsid)
+    return unless Process.respond_to?(:getsid)
+    # OpenBSD doesn't allow Process::getsid(pid) when pid is in
+    # different session.
+    return if /openbsd/ =~ RUBY_PLATFORM
+
+    IO.popen([RUBY, "-e", <<EOS]) do|io|
+	Marshal.dump(Process.getsid, STDOUT)
+	newsid = Process.setsid
+	Marshal.dump(newsid, STDOUT)
+	STDOUT.flush
+	# getsid() on MacOS X return ESRCH when target process is zombie
+	# even if it is valid process id.
+	sleep
+EOS
+      begin
+        # test Process.getsid() w/o arg
+        assert_equal(Marshal.load(io), Process.getsid)
+
+        # test Process.setsid return value and Process::getsid(pid)
+        assert_equal(Marshal.load(io), Process.getsid(io.pid))
+      ensure
+        Process.kill(:KILL, io.pid) rescue nil
+        Process.wait(io.pid)
+      end
+    end
+  end
+
+  def test_spawn_nonascii
+    bug1771 = '[ruby-core:24309] [Bug #1771]'
+
+    with_tmpchdir do
+      [
+       "\u{7d05 7389}",
+       "zuf\u{00E4}llige_\u{017E}lu\u{0165}ou\u{010D}k\u{00FD}_\u{10D2 10D0 10DB 10D4 10DD 10E0 10D4 10D1}_\u{0440 0430 0437 043B 043E 0433 0430}_\u{548C 65B0 52A0 5761 4EE5 53CA 4E1C}",
+       "c\u{1EE7}a",
+      ].each do |name|
+        msg = "#{bug1771} #{name}"
+        exename = "./#{name}.exe"
+        FileUtils.cp(ENV["COMSPEC"], exename)
+        assert_equal(true, system("#{exename} /c exit"), msg)
+        system("#{exename} /c exit 12")
+        assert_equal(12, $?.exitstatus, msg)
+        _, status = Process.wait2(Process.spawn("#{exename} /c exit 42"))
+        assert_equal(42, status.exitstatus, msg)
+        assert_equal("ok\n", `#{exename} /c echo ok`, msg)
+        assert_equal("ok\n", IO.popen("#{exename} /c echo ok", &:read), msg)
+        assert_equal("ok\n", IO.popen(%W"#{exename} /c echo ok", &:read), msg)
+        File.binwrite("#{name}.txt", "ok")
+        assert_equal("ok", `type #{name}.txt`)
+      end
+    end
+  end if windows?
+
+  def test_clock_gettime
+    t1 = Process.clock_gettime(Process::CLOCK_REALTIME, :nanosecond)
+    t2 = Time.now; t2 = t2.tv_sec * 1000000000 + t2.tv_nsec
+    t3 = Process.clock_gettime(Process::CLOCK_REALTIME, :nanosecond)
+    assert_operator(t1, :<=, t2)
+    assert_operator(t2, :<=, t3)
+    assert_raise(Errno::EINVAL) { Process.clock_gettime(:foo) }
+  end
+
+  def test_clock_gettime_constants
+    Process.constants.grep(/\ACLOCK_/).each {|n|
+      c = Process.const_get(n)
+      begin
+        t = Process.clock_gettime(c)
+      rescue Errno::EINVAL
+        next
+      end
+      assert_kind_of(Float, t, "Process.clock_gettime(Process::#{n})")
+    }
+  end
+
+  def test_clock_gettime_GETTIMEOFDAY_BASED_CLOCK_REALTIME
+    n = :GETTIMEOFDAY_BASED_CLOCK_REALTIME
+    t = Process.clock_gettime(n)
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_TIME_BASED_CLOCK_REALTIME
+    n = :TIME_BASED_CLOCK_REALTIME
+    t = Process.clock_gettime(n)
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_TIMES_BASED_CLOCK_MONOTONIC
+    n = :TIMES_BASED_CLOCK_MONOTONIC
+    begin
+      t = Process.clock_gettime(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_GETRUSAGE_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :GETRUSAGE_BASED_CLOCK_PROCESS_CPUTIME_ID
+    begin
+      t = Process.clock_gettime(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_TIMES_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :TIMES_BASED_CLOCK_PROCESS_CPUTIME_ID
+    begin
+      t = Process.clock_gettime(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_CLOCK_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :CLOCK_BASED_CLOCK_PROCESS_CPUTIME_ID
+    t = Process.clock_gettime(n)
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_gettime_MACH_ABSOLUTE_TIME_BASED_CLOCK_MONOTONIC
+    n = :MACH_ABSOLUTE_TIME_BASED_CLOCK_MONOTONIC
+    begin
+      t = Process.clock_gettime(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_gettime(:#{n})")
+  end
+
+  def test_clock_getres
+    r = Process.clock_getres(Process::CLOCK_REALTIME, :nanosecond)
+    assert_kind_of(Integer, r)
+    assert_raise(Errno::EINVAL) { Process.clock_getres(:foo) }
+  end
+
+  def test_clock_getres_constants
+    Process.constants.grep(/\ACLOCK_/).each {|n|
+      c = Process.const_get(n)
+      begin
+        t = Process.clock_getres(c)
+      rescue Errno::EINVAL
+        next
+      end
+      assert_kind_of(Float, t, "Process.clock_getres(Process::#{n})")
+    }
+  end
+
+  def test_clock_getres_GETTIMEOFDAY_BASED_CLOCK_REALTIME
+    n = :GETTIMEOFDAY_BASED_CLOCK_REALTIME
+    t = Process.clock_getres(n)
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    assert_equal(1000, Process.clock_getres(n, :nanosecond))
+  end
+
+  def test_clock_getres_TIME_BASED_CLOCK_REALTIME
+    n = :TIME_BASED_CLOCK_REALTIME
+    t = Process.clock_getres(n)
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    assert_equal(1000000000, Process.clock_getres(n, :nanosecond))
+  end
+
+  def test_clock_getres_TIMES_BASED_CLOCK_MONOTONIC
+    n = :TIMES_BASED_CLOCK_MONOTONIC
+    begin
+      t = Process.clock_getres(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    f = Process.clock_getres(n, :hertz)
+    assert_equal(0, f - f.floor)
+  end
+
+  def test_clock_getres_GETRUSAGE_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :GETRUSAGE_BASED_CLOCK_PROCESS_CPUTIME_ID
+    begin
+      t = Process.clock_getres(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    assert_equal(1000, Process.clock_getres(n, :nanosecond))
+  end
+
+  def test_clock_getres_TIMES_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :TIMES_BASED_CLOCK_PROCESS_CPUTIME_ID
+    begin
+      t = Process.clock_getres(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    f = Process.clock_getres(n, :hertz)
+    assert_equal(0, f - f.floor)
+  end
+
+  def test_clock_getres_CLOCK_BASED_CLOCK_PROCESS_CPUTIME_ID
+    n = :CLOCK_BASED_CLOCK_PROCESS_CPUTIME_ID
+    t = Process.clock_getres(n)
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+    f = Process.clock_getres(n, :hertz)
+    assert_equal(0, f - f.floor)
+  end
+
+  def test_clock_getres_MACH_ABSOLUTE_TIME_BASED_CLOCK_MONOTONIC
+    n = :MACH_ABSOLUTE_TIME_BASED_CLOCK_MONOTONIC
+    begin
+      t = Process.clock_getres(n)
+    rescue Errno::EINVAL
+      return
+    end
+    assert_kind_of(Float, t, "Process.clock_getres(:#{n})")
+  end
+
 end
